#!/bin/sh

#Copyright (c) 2006 SUNY at Stony Brook
#Copyright (c) 2006 Avishay Traeger, Nikolai Joukov, Josef Sipek, and Erez Zadok

BACKUPSCONF=$1

if [ -z $BACKUPSCONF ] ; then
	echo "Usage: crawlbackuprestore.sh backups.conf"
	exit 1
fi

if [ ! -f $BACKUPSCONF ] ; then
	echo "ERROR: backups file does not exist"
	exit 1
fi

source $BACKUPSCONF
FILESTOBACK="${backup[$i]}"
PASSWD="${backup[$i+1]}"
BACKOPTS="${backup[$i+2]}"
IDENT="${backup[$i+3]}"
IDENT=`echo $IDENT | sed s/' '/'%20'/g`

MYAGENT="CrawlBackup"
BACKFILE="backup-$$"
FOUND="0"

#Try Google
OUTFILE="googleres-$$"
wget --quiet -U $MYAGENT -O $OUTFILE http://www.google.com/search?q=%22$IDENT%22
URL=`./parse-search.pl $OUTFILE`
rm -f $OUTFILE
if [ ! -z "$URL" ] ; then
	wget --quiet -U $MYAGENT -O $OUTFILE $URL
	FOUND="1"
fi

#If not, try Yahoo
if [ "$FOUND" = "0" ] ; then
	OUTFILE="yahoores-$$"
	wget --quiet -U $MYAGENT -O $OUTFILE http://search.yahoo.com/search?p=%22$IDENT%22
	URL=`./parse-search.pl $OUTFILE`
	rm -f $OUTFILE
	if [ ! -z "$URL" ] ; then
		wget --quiet -U $MYAGENT -O $OUTFILE $URL
		FOUND="1"
	fi
fi

if [ "$FOUND" = "0" ] ; then
	echo "ERROR: Could not find cached copy of data."
	exit 1
fi

#Get the options
BACKOPTS=`grep -B 2 "==-== FILES ==-==" $OUTFILE | head -n 1`
DECRYPT=`echo $BACKOPTS | grep -c enc`
DECOMP=`echo $BACKOPTS | grep -c comp`

#Get the data
TMPFILE="tmpfile-$$"
LINEBEG=`grep -n "==-== DATA ==-==" $OUTFILE | cut -d : -f 1`
LINEBEG=`expr $LINEBEG + 1`
LINEEND=`grep -n "==-== CRAWLBACKUPRESTORE ==-==" $OUTFILE | cut -d : -f 1`
LINEEND=`expr $LINEEND - 2`
NUMLINES=`expr $LINEEND - $LINEBEG + 1`
grep -A $NUMLINES "==-== DATA ==-==" $OUTFILE | tail -n $NUMLINES > $TMPFILE

#Compare the checksums
SHA1ORG=`grep -A 1 "\[SHA1SUM\]" $OUTFILE | tail -n 1 | cut -d ' ' -f 1`
SHA1NEW=`sha1sum -b $TMPFILE | cut -d ' ' -f 1`
if [ "$SHA1ORG" != "$SHA1NEW" ] ; then
	echo "WARNING: Checksum mismatch!  Trying to continue..."
fi

CPWD=`pwd`
fn="$CPWD/$TMPFILE"
RESTOREROOT="/tmp/$$.$RANDOM.$RANDOM"
mkdir $RESTOREROOT
cd $RESTOREROOT
	
echo "Using $RESTOREROOT as restoration directory"
	
if [ "$DECRYPT" = "1" ] ; then
	gpg --decrypt < $fn > dec
else
	uudecode -o dec < $fn
fi

if [ "$DECOMP" = "1" ] ; then
	file="dec.$RANDOM.tar.gz"
	mv dec $file
	tarflags="xzf"
else
	file="dec.$RANDOM.tar"
	mv dec $file
	tarflags="zf"
fi

tar $tarflags $file
rm -f $fn
rm -f $file
