#!/bin/sh

#Copyright (c) 2006 SUNY at Stony Brook
#Copyright (c) 2006 Avishay Traeger, Nikolai Joukov, Josef Sipek, and Erez Zadok

ACCOUNTSCONF=$1
BACKUPSCONF=$2
RESTOREFILE=$3
RESTOREDATE=$4

source common

if [ -z $BACKUPSCONF ] ; then
	echo "Usage: mailrestore.sh accounts.conf backups.conf filename [date]"
	exit 1
fi
source $BACKUPSCONF

if [ ! -f $ACCOUNTSCONF ] ; then
	echo "ERROR: accounts file does not exist"
	exit 1
fi
source $ACCOUNTSCONF

if [ -z "$RESTOREFILE" ] ; then
	echo "ERROR: specify file to be restored"
	exit 1
fi

if [ ! -z "$RESTOREDATE" ] ; then
	RESTOREDATE=`date --date "$RESTOREDATE" 2> /dev/null`
	if [ $? -ne 0 ] ; then
		echo "ERROR: invalid date format"
		exit 1
	fi
else
	RESTOREDATE=`date`
fi

echo "Using \$RESTOREDATE = $RESTOREDATE"

function possible_accounts()
{
	TWANT=`date +%s --date "$RESTOREDATE"`
	
	NUMBACKELEM=${#backup[@]}
	NUMACCOUNTELEM=${#accounts[@]}
	i=0
	while [ "$i" -lt "$NUMBACKELEM" ]
	do
		THAVE=`date +%s --date "${backup[$i+3]}"`

		if [ $THAVE -ge $TWANT ] ; then
			# too new for us
			continue;
		fi

		# That's right, we rewrite the glob into a regular
		# expression
		GLOBLIST=`echo ${backup[$i+1]} | sed \
			-e "s,;, ,g" \
			-e "s,\.,\\\.,g" \
			-e "s,\*,\[\^/\]\*,g"`

		for glob in $GLOBLIST
		do
			l=`echo $RESTOREFILE | grep -e "$glob" | wc -l`
			if [ $l -ne 0 ] ; then
				echo ${backup[$i]}
				eecho "$glob matches ($l)"
				break
			fi
		done

		i=`expr $i + $ELEMINBACKCONF`
	done
}

paccs=`possible_accounts $ACCOUNTSCONF $BACKUPSCONF $RESTOREFILE $RESTOREDATE`
for pacc in $paccs
do
	echo "trying $pacc..."

	MBOX="$RANDOM.mbox"
	POPSERVER="pop3.seznam.cz"
	FLAGS="--ssl"
	fetchmail -k -a -p POP3 $FLAGS -m "./writemail.py $MBOX %F %T" -u $pacc $POPSERVER

	# "f=v;f=v file"
	tmp=`./parsembox.py $MBOX $RESTOREFILE`
	flags=`echo $tmp | awk '{print $1}'`
	fn=`echo $tmp | cut -d ' ' -f 2-`

	CPWD=`pwd`
	fn="$CPWD/$fn"
	RESTOREROOT="/tmp/$$.$RANDOM.$RANDOM"
	mkdir $RESTOREROOT
	cd $RESTOREROOT
	
	echo "Using $RESTOREROOT as restoration directory"
	
	if [ `echo $flags | grep ENCRYPTED | wc -l` -eq 1 ] ; then
		# encrypted
		gpg --decrypt < $fn > dec
	else
		# uuencoded
		uudecode -o dec < $fn
	fi

	if [ `echo $flags | grep COMPRESSED | wc -l` -eq 1 ] ; then
		# compressed
		file="dec.$RANDOM.tar.gz"
		mv dec $file
		tarflags="xzf"
	else
		# uncompressed
		file="dec.$RANDOM.tar"
		mv dec $file
		tarflags="zf"
	fi

	tar $tarflags $file
done

